/* supportp.c

   S1 cpu simulator
   Computer Architecture 1997

   supporting function

   P. Chongstitvatana
   Department of Computer Engineering
   Chulalongkorn University

   pipeline   15 Aug 97

*/

setbusy(n)   /* set register access to busy */
int n;
{
  Rbusy[n] = 1;
}

isbusy(n)    /* check busy status of a register */
int n;
{
  return( Rbusy[n] );
}

clearallbusy()
{
  int i;
  for( i=0; i<8; i++ ) Rbusy[i] = 0;
}

report()
{
  float cpi,p;
  if (clock == 0 ) return;
  cpi = (float) (clock+stall)/clock;
  p = 100.0 * (float)stall/(clock+stall);
  printf("%d instructions, stall %d clocks %5.2f %%, CPI = %5.2f\n",
	 clock,stall,p,cpi);
}

/* IR decode */
IRop(ir) int ir;  { return( ( ir & 0xE000 ) >> 13 ); } /* bit 15..13 */
IRr(ir) int ir;   { return( ( ir & 0x1C00 ) >> 10 ); } /* bit 12..10 */
IRads(ir) int ir; { return(   ir & 0x03FF         ); } /* bit 9..0   */
IRxop(ir) int ir; { return( ( ir & 0x1E00 ) >> 9  ); } /* bit 12..9  */
IRr1(ir) int ir;  { return( ( ir & 0x0038 ) >> 3  ); } /* bit 5..3   */
IRr2(ir) int ir;  { return(   ir & 0x0007         ); } /* bit 2..0   */

int decodeIR() /* 0..6, 7..12 */
{
  if( IRop(Ir) == 7 )
    return(7 + IRxop(Ir));
  else
    return(IRop(Ir));
}

trap( int ads )			/* special function */
{
  switch( ads ) {
  case 1000 : if( core ) dump(start,nbyte);
	      report();
	      exit(0); 		    /* stop */
  case 1001 : printf("%d\n",R[0]);  /* print R0 */
	      break;
  default: return 0;
  }
  return 1;
}

/* load object code from file */

geta(fp)	/* get one charactor */
FILE *fp;	/* skip newline and blank */
{
  char s[32];
  fscanf(fp,"%s",s);
  return s[0] ;
}

geti(fp)	/* get one integer */
FILE *fp;
{
  int c;
  fscanf(fp,"%d",&c);
  return c;
}

/* set maximum of number to be read to 200
   to prevent infinite loop when input is malformed
   the input file name is fixed to "in.obj"
*/
loadprogram()
{
  int cnt,ip,c,c1,c2;
  FILE *fp;

  fp = fopen("in.obj","r");
  ip = 0;
  for(cnt=0; cnt<200; cnt++ ) {		/* max 200 words */
    switch( geta(fp) ) {
    case 'a': ip = geti(fp);		/* set address */
	break;
    case 'i': c = geti(fp);		/* instruction */
	c1 = geti(fp);
	c2 = geti(fp);
	if(c != 7)
	  M[ip] = (c << 13) | (c1 << 10) | c2;
	else
	  M[ip] = (c << 13) | (c1 << 9) | (c2 << 3) | geti(fp);
	ip++;
	break;
    case 'w': M[ip] = geti(fp);		/* set constant data */
	ip++;
	break;
    case 't': trace = 1;		/* trace mode */
	break;
    case 'd': core = 1;                 /* dump start nbyte */
	start = geti(fp);
	nbyte = geti(fp);
	break;
    case 'e': cnt = 1000;		/* end */
	break;
    }
  }
  fclose(fp);
  printf("load program, last address %d\n",ip);
}

dump(ads,n)	/* dump n words starting from ads */
int ads,n;
{
  int i;
  printf(" ads   dec\n");
  for(i=ads; i<ads+n; i++)
    printf("%4d %6d\n",i,M[i]);
}

show()		/* show some registers */
{
  int op, ox;
  op = IRop(Ir);
  if( op == 7 ) ox = IRxop(Ir); else ox = 0;
  printf("PC %4d op %d xop %d R0 %d R1 %d R2 %d R7 %d Z %d S %d\n",
	Pc,op,ox,R[0],R[1],R[2],R[7],Z,S);
}
